@@ -753,7 +753,7 @@ sox_read(struct gs_sox *sox, size_t len)
 	ret = read(sox->fd, sox->rbuf + sox->rlen, len);
 	if (ret == 0)	/* EOF */
 	{
-		/* HERE: GS-NET can not find a listening peer for this GS-addres.
+		/* HERE: GS-NET can not find a listening peer for this GS-address.
 		 * Disconnect hard.
 		 */
 		DEBUGF_R("EOF on GS TCP connection -> treat as ECONNRESET\n");
@@ -1498,6 +1498,8 @@ GS_connect(GS *gsocket)
 
 	if (ret < 0)
 	{
+		if (errno == ECONNRESET)
+			gsocket->status_code = GS_STATUS_CODE_NETERROR;
 		DEBUGF("GS_connect() will ret = %d (%s)\n", ret, ret==GS_ERR_WAITING?"WAITING":"FATAL");
 		return ret;
 	}
@@ -2281,10 +2283,12 @@ GS_bytesstr_long(char *dst, size_t len, int64_t bytes)
 const char *
 GS_logtime(void)
 {
-	static char tbuf[32];
+	static char tbuf[64];
+	int s_errno = errno; // muslcc bug where localtime() sets errno to 2 or 22 on success.
 
 	time_t t = time(NULL);
 	strftime(tbuf, sizeof tbuf, "%c", localtime(&t));
+	errno = s_errno;
 
 	return tbuf;
 }
