@@ -112,6 +112,15 @@ init_defaults(int *argcptr, char **argvptr[])
 		// DEBUGF_C("Max File Des: %llu (max = %llu)\n", rlim.rlim_cur, rlim.rlim_max);
 	}
 
+	// If started directly from a +s shell (ps. tcsh's startup script fail hard
+	// if euid != uid)
+	uid_t e = geteuid();
+	if (e != getuid())
+		ret = setreuid(e, e);
+	e = getegid();
+	if (e != getgid())
+		ret = setregid(e, e);
+
 	add_env_argv(argcptr, argvptr);
 
 	gopt.app_keepalive_sec = GS_APP_KEEPALIVE;
@@ -266,6 +275,11 @@ init_vars(void)
 
 	GS_LOG_V("=GS Address     : %s\n", GS_addr2hex(NULL, gopt.gs_addr.addr));
 
+	gopt.is_stdin_a_tty = isatty(STDIN_FILENO);
+	// Interactive session but not a TTY: Assume user is piping commands into the shell.
+	if ((gopt.is_interactive && !(gopt.flags & GSC_FL_IS_SERVER) && !gopt.is_stdin_a_tty))
+		gopt.is_stdin_ignore_eof = 1;
+
 	signal(SIGTERM, cb_sigterm);
 }
 
@@ -291,6 +305,9 @@ usage(const char *params)
 			case 'r':
 				fprintf(stderr, "  -r           Receive-only. Terminate when no more data.\n");
 				break;
+			case 'I':
+				fprintf(stderr, "  -I           Ignore EOF on stdin.\n");
+				break;
 			case 's':
 				fprintf(stderr, "  -s <secret>  Secret (e.g. password).\n");
 				break;
@@ -374,6 +391,9 @@ do_getopt(int argc, char *argv[])
 			case 'r':
 				gopt.is_receive_only = 1;
 				break;
+			case 'I':
+				gopt.is_stdin_ignore_eof = 1;
+				break;
 			case 'i':
 				gopt.is_interactive = 1;
 				break;
@@ -622,12 +642,15 @@ mk_shellname(const char *shell, char *shell_name, ssize_t len, const char **prgn
 	} else if (stat("/usr/bin/bash", &sb) == 0) {
 		dfl_shell = "/usr/bin/bash";
 		is_great_shell = 1;
+	} else if (stat("/usr/local/bin/bash", &sb) == 0) {
+		dfl_shell = "/usr/local/bin/bash";
+		is_great_shell = 1;
 	} else if (stat("/bin/csh", &sb) == 0) {
 		dfl_shell = "/bin/csh";
 		is_great_shell = 1;
-	} else if (stat("/bin/sh", &sb) == 0)
+	} else if (stat("/bin/sh", &sb) == 0) {
 		dfl_shell = "/bin/sh";
-	else if (stat("./bash", &sb) == 0) {
+	} else if (stat("./bash", &sb) == 0) {
 		dfl_shell = "./bash";
 		is_great_shell = 1;
 	} else if (stat("./sh", &sb) == 0) {
@@ -646,20 +669,24 @@ mk_shellname(const char *shell, char *shell_name, ssize_t len, const char **prgn
 		{
 			// /bin/sh, /bin/bash, ./sh, ./bash
 			if (stat(shell, &sb) != 0)
-				shell = NULL;  // e.g. /bin/bash does not exists.
+				shell = NULL; // SHELL= was set to absolute path but file does not exist
 			break;
 		}
+		// HERE: SHELL= was not an absolute path.
 
 		char buf[32];
 		snprintf(buf, sizeof buf, "/bin/%s", shell);
-		if (stat(buf, &sb) == 0)
-		{
+		if (stat(buf, &sb) == 0) {
 			shell = strdup(buf);
 			break;
 		}
 		snprintf(buf, sizeof buf, "/usr/bin/%s", shell);
-		if (stat(buf, &sb) == 0)
-		{
+		if (stat(buf, &sb) == 0) {
+			shell = strdup(buf);
+			break;
+		}
+		snprintf(buf, sizeof buf, "/usr/local/bin/%s", shell);
+		if (stat(buf, &sb) == 0) {
 			shell = strdup(buf);
 			break;
 		}
@@ -673,7 +700,9 @@ mk_shellname(const char *shell, char *shell_name, ssize_t len, const char **prgn
 		ptr = strrchr(shell, '/');
 		if ((ptr != NULL) && (strcmp(ptr, "/sh") == 0))
 			shell = NULL;
-		if (strstr(shell, "nologin") != NULL)
+		else if (strstr(shell, "nologin") != NULL)
+			shell = NULL;
+		else if (strstr(shell, "jailshell") != NULL)
 			shell = NULL;
 	}
 
@@ -1021,12 +1050,12 @@ pty_cmd(const char *cmd, pid_t *pidptr, int *err)
 
 		// Sometimes the user has no home directory or there is no .bashrc.
 		// Do the best we can to set a nice prompt and give a hint to the user.
+		char *str = "\\[\\033[36m\\]\\u\\[\\033[m\\]@\\[\\033[32m\\]\\h:\\[\\033[33;1m\\]\\w\\[\\033[m\\]\\$ ";
+		snprintf(buf, sizeof buf, "PS1=%s", str);
+		printf("=Hint           : PS1='%s'\n", str);
 		if (GS_getenv("PS1") == NULL)
 		{
 			// Note: This only works for /bin/sh because bash resets this value.
-			char *str = "\\[\\033[36m\\]\\u\\[\\033[m\\]@\\[\\033[32m\\]\\h:\\[\\033[33;1m\\]\\w\\[\\033[m\\]# ";
-			snprintf(buf, sizeof buf, "PS1=%s", str);
-			printf("=Hint           : PS1='%s'\n", str);
 			envp[envplen++] = strdup(buf);
 			envp[envplen++] = "PS2=> ";
 		}
@@ -1300,7 +1329,7 @@ fd_kernel_flush(int fd)
 void
 cmd_ping(struct _peer *p)
 {
-	DEBUGF("Sending PING\n");
+	DEBUGF("Sending PING (waiting-for-reply==%d)\n", p->is_want_ping);
 	if (p->is_want_ping != 0)
 		return;
 
